/*****************************************************************************
 * eit.h: EIT structures
 *----------------------------------------------------------------------------
 * (c)2001-2002 VideoLAN
 * $Id: eit.h,v 1.1 2002/01/22 20:30:16 bozo Exp $
 *
 * Authors: Damien Delhomme <delhomd4@cti.ecp.fr>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *----------------------------------------------------------------------------
 *
 *****************************************************************************/

#ifndef _DVBPSI_EIT_H_
#define _DVBPSI_EIT_H_


/*****************************************************************************
 * dvbpsi_eit_event_t
 *****************************************************************************
 * This structure is used to store a decoded EIT event.
 * (ISO/IEC 13818-1 section 2.4.4.3). ???????????????????????????????
 *****************************************************************************/

typedef struct dvbpsi_eit_event_s
{
  uint16_t                  i_event_id;
  uint40_t                  i_start_time;
  uint24_t                  i_duration;
  uint8_t                   i_running_status;
  int			    b_free_CA_mode;

  dvbpsi_descriptor_t *     p_first_descriptor; /* descriptor list */

} dvbpsi_eit_event_t;

/*****************************************************************************
 * dvbpsi_eit_t
 *****************************************************************************
 * This structure is used to store a decoded EIT.
 * (ISO/IEC 13818-1 section 2.4.4.3). ????????????????????????????????
 *****************************************************************************/
typedef struct dvbpsi_eit_s
{
  uint16_t                  i_service_id;
  uint8_t                   i_version;          /* version_number */
  int                       b_current_next;     /* current_next_indicator */
  uint16_t		    i_ts_id;		/* transport_stream_id */

  dvbpsi_eit_event_t *      p_first_event;

} dvbpsi_eit_t;


/*****************************************************************************
 * dvbpsi_eit_callback
 *****************************************************************************
 * Callback type definition.
 *****************************************************************************/
typedef void (* dvbpsi_eit_callback)(void* p_cb_data, dvbpsi_eit_t* p_new_eit);


/*****************************************************************************
 * dvbpsi_AttachEIT
 *****************************************************************************
 * Initialize a EIT decoder and return a handle on it.
 *****************************************************************************/
dvbpsi_handle dvbpsi_AttachEIT(dvbpsi_eit_callback pf_callback,
                               void* p_cb_data);


/*****************************************************************************
 * dvbpsi_DetachEIT
 *****************************************************************************
 * Close a EIT decoder. The handle isn't valid any more.
 *****************************************************************************/
void dvbpsi_DetachEIT(dvbpsi_handle h_dvbpsi);


/*****************************************************************************
 * dvbpsi_InitEIT/dvbpsi_NewEIT
 *****************************************************************************
 * Initialize a pre-allocated/new dvbpsi_eit_t structure.
 *****************************************************************************/
void dvbpsi_InitEIT(dvbpsi_eit_t* p_eit, uint16_t i_service_id,
		    uint8_t i_version, int b_current_next, uint16_t i_ts_id);



#define dvbpsi_NewEIT(p_eit, i_service_id, i_version,
		      b_current_next, i_ts_id)
  p_eit = (dvbpsi_eit_t*)malloc(sizeof(dvbpsi_eit_t));
  if(p_eit != NULL)
    dvbpsi_InitEIT(p_eit, i_service_id, i_version,
                      b_current_next, i_ts_id);


void dvbpsi_EmptyEIT(dvbpsi_eit_t* p_eit);

#define dvbpsi_DeleteEIT(p_eit)
  dvbpsi_EmptyEIT(p_eit);
  free(p_eit);


dvbpsi_descriptor_t* dvbpsi_EITAddDescriptor(dvbpsi_eit_t* p_eit,
                                             uint8_t i_tag, uint8_t i_length,
                                             uint8_t* p_data);


dvbpsi_eit_event_t* dvbpsi_EITAddEvent(dvbpsi_eit_t* p_eit, uint16_t i_event_id,
				       uint40_t i_start_time,
				       uint24_t i_duration,
				       uint8_t i_running_status,
				       int b_free_CA_mode);



dvbpsi_descriptor_t* dvbpsi_EITEventAddDescriptor(dvbpsi_eit_es_t* p_es,
                                                  uint8_t i_tag, uint8_t i_length,
                                                  uint8_t* p_data);


dvbpsi_psi_section_t* dvbpsi_GenEITSections(dvbpsi_eit_t* p_eit);


